local config = require("gemini.config")
local http = require("gemini.http")
local chat = require("gemini.chat")  -- Add chat module
local M = {}

local function get_api_key()
	return vim.g.gemini_api_key or os.getenv("GEMINI_API_KEY")
end

local function create_contents(prompt, context)
	local contents = {}
	
	-- Add conversation history
	for _, message in ipairs(chat.get_conversation_history()) do
		table.insert(contents, {
			role = message.role,
			parts = {{text = message.content}}
		})
	end

	-- Add current prompt
	if context then
		table.insert(contents, {
			role = "user",
			parts = {{text = "Context:\n" .. context .. "\n\nQuery: " .. prompt}}
		})
	else
		table.insert(contents, {
			role = "user",
			parts = {{text = prompt}}
		})
	end

	-- Ensure we have at least one content item
	if #contents == 0 then
		contents = {
			{
				role = "user",
				parts = {{text = prompt}}
			}
		}
	end

	return contents
end

local function store_message(role, content)
	chat.add_message(role, content)
end

local function handle_response(result, callback)
	if result.error then
		callback(nil, "API Error: " .. vim.inspect(result.error))
		return
	end

	if result.candidates and
	   result.candidates[1] and
	   result.candidates[1].content and
	   result.candidates[1].content.parts and
	   result.candidates[1].content.parts[1] and
	   result.candidates[1].content.parts[1].text then
		
		local response_text = result.candidates[1].content.parts[1].text
		store_message("model", response_text)
		callback(response_text)
	else
		callback(nil, "Unexpected response structure")
	end
end

function M.get_response(prompt, context, callback)
	local api_key = get_api_key()
	
	if not api_key then
		vim.schedule(function()
			callback(nil, "API key not set")
		end)
		return
	end

	-- Validate prompt
	if not prompt or prompt:match("^%s*$") then
		vim.schedule(function()
			callback(nil, "Empty prompt")
		end)
		return
	end

	-- Create the request content
	local contents = {
		{
			role = "user",
			parts = {{text = context and (context .. "\n\n" .. prompt) or prompt}}
		}
	}

	local payload = vim.json.encode({
		contents = contents,
		generationConfig = {
			temperature = 0.1,
			topK = 1,
			topP = 1,
			maxOutputTokens = 2048,
		}
	})
	
	-- Correctly format the URL with model and API key
	local url = string.format(
		config.options.api_url,
		config.options.model
	) .. "?key=" .. api_key

	local request = http.Request.new(url, payload)
	request:execute(function(result, error)
		if error then
			vim.schedule(function()
				callback(nil, "HTTP Error: " .. error)
			end)
			return
		end
		
		handle_response(result, callback)
	end)
end

function M.clear_conversation()
	chat.clear()  -- This will clear both the buffer and conversation history
end

return M
